#!/usr/bin/env python3
import importlib

class Buffer: # all the information about the current file
	def carat(self):
		return self.index
	def dollar(self):
		return self.length() - 1 + self.index

	# Rather than get the content as it's presented in the buffer object
	# (as one big string), get it as a list.
	def content_list(self):
		return self.content.rstrip(self.delimiter).split(self.delimiter)

	# Build a string with the same format as the buffer.content and set the
	# buffer.content to that string, from the kind of list output by
	# buffer.content_list().
	# buffer.content_set_list(buffer.content_list()) is an expensive nop.
	def content_set_list(self, content_as_list):
		content = ""
		for line in content_as_list:
			content += line + self.delimiter
		self.content = content
		return None

	# this is really bad because any module can call import_module_ just as
	# easily as any other. so malicious modules that, say, take 'q' and
	# make it upload a file to some external server before exiting would be
	# super easy to make.
	# the solution I see is OS-level permissions but this needs to be
	# talked about like all the time if this tool gets popular
	# [why would it?] lest some fool runs a zelda.sh script that changes
	# ~/src/it/w.py and doesn't realize in time
	def import_module_(self, name):
		try:
			self.modules[name] = importlib.import_module(name)
		except (ModuleNotFoundError, TypeError) as err:
			print(err)
			return False
		else:
			return True

	def length(self):
		return self.content.count(self.delimiter)

	def __init__(self):
		self.content = ""         # content of the file
		self.delimiter = "\n"     # line delimiter
		self.env = {}             # configuration dictionary
		self.filename = ""        # name of where we'll save the file
		self.index = 0            # indexing of dot
		self.dot = self.index - 1 # invalid position to start
		self.modules = {}         # see it.py
		self.saved = 1            # is buffer saved?

def main(buffer, command):
	if len(command) == 1:
		command += list(vars(buffer))
	for attrib in command[1:]:
		if attrib in list(vars(buffer)):
			val = vars(buffer)[attrib]
			# so self.modules shows as <class 'dict'>
			if not(type(val) is int or type(val) is str):
				val = str(type(val))
			# special case usually for self.delimiter
			elif val == "\n":
				val = "<newline>"
			# only affects it if type(val) is int
			else:
				val = str(val)
			print("%s:\t%s" % (attrib, val))
		else:
			print("No attribute: %s\n" % attrib, end='')
	return buffer
